package CAConfig;
use strict;

my $isinit;
my $configvars = {};
my $datadirfile = 'datadir.dat';
my $configfile = 'caconfig.dat';
my $lasterror;
my $errormessages = {
  NoError => 'No Errors',
  NoSuchError => 'InternalError: Request for undefined error message',
  DataDirErr => 'Failed to open DATADIR info file',
  NeedSetup => 'Package is not installed, use Setup command',
  CfgOpenErr => 'Failed to open CONFIG file for reading',
  CfgWriteErr => 'Failed to update CONFIG file',
};
my $defaults = {
 sidfile => 'casession',
 logfile => 'calog.dat',
 cntfile => 'cacount.txt',
 cacache => 'idcache.dat',
 cadata => 'cadata.dat',
 imgdir => '.',
# Name for Campaign Analyzer cookie. You may need to change the default value
# if another application installed on your site uses the same cookie name.
 cookname => 'CampaignAnalyzer',
 cachewindow => 604800,
 cachepurge => 18144000,
# The value for domain attribute as per Set-Cookie HTTP header. If no value
# provided Campaign Analyzer will use your site domain name as per HTTP_HOST
# environment variable (the 'www' part if any will be truncated).
#
# Example:
# cookdomain => 'mysite.com',
};

# ======================================================================= errmsg

sub errmsg {
  my $i = $_[0];
  $i = 'NoError' if $i eq undef;
  $i = 'NoSuchError' if ! exists $errormessages->{$i};
  return $errormessages->{$i}
}
# ======================================================================== error

sub error {
  my $e = $lasterror;
  $lasterror = undef;
  return $e;
}
# ====================================================================== cfgfile

sub cfgfile {
  return $configfile;
}
# ======================================================================== dfile

sub dfile {
  return $datadirfile;
}
# ====================================================================== getvars

sub getvars {
  return $configvars;
}
# ========================================================================= init
sub init {
  return 1 if $isinit;
  my $datadir = scriptdir();
  my $df = "$datadir/$datadirfile";
  $configvars->{datadir} = $datadir;
  if (-e $df) {
    if (!open(DATADIR, $df)) {
      $lasterror = 'DataDirErr';
      return 0;
    }
    $datadir = <DATADIR>;
    close DATADIR;
    $datadir =~ s/\r//g;
    chomp($datadir);
    $configvars->{datadir} = $datadir;
  }
  my $cf = "$datadir/$configfile";
  if (! -e $cf) {
    $lasterror = 'NeedSetup';
    foreach (keys %{$defaults}) {
      $configvars->{$_} = $defaults->{$_} if ! exists $configvars->{$_};
    }
    return 0;
  }
  if (!open(CFG, $cf)) {
    $lasterror = 'CfgOpenErr';
    return 0;
  }
  my @buff = <CFG>;
  chomp @buff;
  close CFG;
  foreach (@buff) {
    my @parts = split(/=/);
    my $name = shift @parts;
    next if $name eq undef;
    $configvars->{$name} = join('=', @parts);
  }
  foreach (keys %{$defaults}) {
    $configvars->{$_} = $defaults->{$_} if ! exists $configvars->{$_};
  }
  $isinit = 1;
  return 1;
}
# ========================================================================= save

sub save {
  my $cf = $configvars->{datadir}."/$configfile";
  if (!open(CFG, ">$cf")) {
    $lasterror = 'CfgWriteErr';
    return 0;
  }
  eval "flock(CFG, 2)";
  foreach (keys %{$configvars}) {
    next if $_ eq 'datadir';
    print CFG $_."=".$configvars->{$_}."\n";
  }
  if (!close(CFG)) {
    $lasterror = 'CfgWriteErr';
    return 0;
  }
  return 1;
}
# ==================================================================== scriptdir

sub scriptdir {
  my $dir = $ENV{'SCRIPT_FILENAME'};
  $dir = $ENV{'PATH_TRANSLATED'} if $ENV{'PATH_TRANSLATED'} ne undef;
  return $ENV{'PWD'} if $dir eq undef;
  my @path = split(/\/|\\/, $dir);
  pop(@path);
  $dir = join("\/", @path);
  return $dir;
}

1;