package CReporter;
use strict;
use CAConfig;

my $data = {};
my $lasterror;
my $cfgvars = {};
my $logfile;
my $ignorevisitors = {};
my $reporttime;
# ===================================================================== datasave
# datasave(item, countall, $starttm, $endtm)

sub datasave {
  $reporttime = time();
  my $dftmp = $cfgvars->{datadir}."/".$cfgvars->{cadata};
  if (!open(DFILE, ">$dftmp")) {
    $lasterror = "Failed to create temporary file";
    return 0;
  }
  eval "flock(DFILE, 2)";
  my $item = ($_[0] || '-');
  my $countall = ($_[1] || '0');
  print DFILE "$item $countall $_[2] $_[3]\n$reporttime\n";
  while ( my ($key, $ptr) = each %{$data}) {
    print DFILE "$key\t".($ptr->{HIT} || '0')."\t".($ptr->{LEAD} || '0')."\t".($ptr->{SALE} || '0')."\n";
  }
  if (!close(DFILE)) {
    $lasterror = "Failed to close temporary file";
    return 0;
  }
  return 1;
}
# ===================================================================== dataload
# dataload(item, countall, $starttm, $endtm)

sub dataload {
  my $item = ($_[0] || '-');
  my $countall = ($_[1] || '0');
  my $dftmp = $cfgvars->{datadir}."/".$cfgvars->{cadata};
  return 0 if !open(DFILE, $dftmp);
  eval "flock(DFILE, 2)";
  my $line = <DFILE>;
  if ($line ne "$item $countall $_[2] $_[3]\n") {
    close DFILE;
    return 0;
  }
  $reporttime = <DFILE>;
  chomp $reporttime;
  foreach (<DFILE>) {
    chomp $_;
    my ($source, $hit, $lead, $sale) = split(/\t/);
    $data->{$source}->{HIT} = $hit;
    $data->{$source}->{LEAD} = $lead;
    $data->{$source}->{SALE} = $sale;
  }
  close DFILE;
  return 1;
}
# ======================================================================== _init

sub _init() {
  $cfgvars = CAConfig::getvars();
  $logfile = $cfgvars->{datadir}."/".$cfgvars->{logfile};
  my @ignorelist = split(/;/, $cfgvars->{ignorevisitors});
  foreach (@ignorelist) { $ignorevisitors->{$_} = 1; }
}
# ======================================================================== error

sub error {
  my $e = $lasterror;
  $lasterror = undef;
  return $e;
}
# ======================================================================== rtime

sub rtime {
  return localtime($reporttime);
}
# ======================================================================= report
# report(item, countall, $starttm, $endtm)

sub report {
  _init();
  if (! -e $logfile) {
    $lasterror = "Log file does not exist (did you add tracking code?)";
    return undef;
  }
  if (!$_[4]) {
    return $data if dataload($_[0], $_[1], 0, 0);
  }
  if (!open(LOG, $logfile)) {
    $lasterror = "Failed to open log file";
    return undef;
  }
  for (my $line = <LOG>; $line ne undef; $line = <LOG>) {
    chop($line);
    my ($visitor, $event, $item, $tm, $ip, $page, $rfr) = split(/\s+/, $line);
    next if $event eq '-';
    next if exists $ignorevisitors->{$visitor};
    if ($event eq 'HIT') { # There is only one HIT event per visitor
      $data->{sourceis($page, $rfr)}->{HIT}++;
      next;
    }
    next if $_[0] ne undef && $item ne $_[0];
    $data->{Unknown}->{$event}++;
  }
  close LOG;
  datasave($_[0], $_[1], 0, 0);
  return $data;
}
# =================================================================== sourcedata
# sourcedata(sourceid)

sub sourcedata {
  my $data;
  _init();
  if (! -e $logfile) {
    $lasterror = "Log file does not exist (make sure you completed setup)";
    return undef;
  }
  if (!open(LOG, $logfile)) {
    $lasterror = "Failed to open log file";
    return undef;
  }
  for (my $line = <LOG>; $line ne undef; $line = <LOG>) {
    chop($line);
    my ($visitor, $event, $item, $tm, $ip, $page, $rfr) = split(/\s+/, $line);
    next if $event ne 'HIT';
    next if $rfr eq undef;
    my $rfrurl = $rfr;
    next if sourceis($page, $rfr) ne $_[0];
    $data->{$rfrurl}++;
  }
  close LOG;
  return $data;
}
# ===================================================================== sourceis

sub sourceis {
  my ($url, $query) = split(/\?/, $_[0]);
  if ($query ne undef) {
    my @pairs = split(/&/, $query);
    foreach my $pair (@pairs) {
      my ($name, $value) = split(/=/, $pair);
      next if $name !~ /[Ss][Oo][Uu][Rr][Cc][Ee]/;
      next if $value eq undef;
      $value =~ tr/+/ /;
      $value =~ s/%([a-fA-F0-9][a-fA-F0-9])/pack("C", hex($1))/eg;
      return $value;
    }
  }
  if ($_[1] ne undef) {
    if ($_[1] =~ /^[^\/]+:\/\/(?:www\.)?([^\/]+)/) { return $1; }
  }
  return 'Unknown';
}
1;